#include "..\common\common.h"

wmain(int argc, WCHAR *argv[]) {

	HANDLE hFile;
	WCHAR wstrFileName[4+BUFSIZE] = L"\\\\?\\";

	/* Argument Check */
	if (argc != 2) {
		fprintf(stderr, "Usage: %S FileName[:StreamName]\n", argv[0]);
		fprintf(stderr, "       FileName can consist of the file name (up to 255 Unicode chars) \n");
		fprintf(stderr, "       and StreamName (up to 255 Unicode chars) \n");
		fprintf(stderr, "       Write to file via stdin\n");
		ExitProcess(1);
	}

	/* generate full path name */
	{
		WCHAR *lpwstrFileName;
		if (GetFullPathNameW(argv[1], BUFSIZE, wstrFileName+4, &lpwstrFileName) == 0) {
			PrintErrorMessage(__FILE__, "GetFullPathNameW", __LINE__);
			ExitProcess(1);
		}
	}

	hFile = CreateFileW(wstrFileName,
        GENERIC_WRITE,  // open for reading 
        0,
        NULL,                      // no security 
        OPEN_ALWAYS,               // create if the file does not exist
        FILE_ATTRIBUTE_NORMAL,     // normal file 
        NULL);                     // no attr. template 

	if (hFile == INVALID_HANDLE_VALUE) { 
		PrintErrorMessage(__FILE__, "CreateFile", __LINE__);
		ExitProcess(1);
	}

	{
		BOOL bResult;
		DWORD nBytesRead;
		HANDLE hStdin;
	
		hStdin = GetStdHandle(STD_INPUT_HANDLE);
		if (hStdin == INVALID_HANDLE_VALUE) { 
			PrintErrorMessage(__FILE__, "GetStdHandle", __LINE__);
			ExitProcess(1);
		}

		do {
			CHAR inBuffer[256] = {0};
			DWORD nBytesToRead = 256;
			if (bResult = ReadFile(hStdin, &inBuffer, nBytesToRead, &nBytesRead, NULL)) {
				DWORD nBytesWritten;
				DWORD nBytesToWrite = nBytesRead;
				while (nBytesToWrite > 0) {
					if (!WriteFile(hFile, &inBuffer, nBytesToWrite, &nBytesWritten, NULL)) {
						PrintErrorMessage(__FILE__, "WriteFile", __LINE__);
						ExitProcess(1);
					}
					nBytesToWrite = nBytesToWrite - nBytesWritten;
				}
			}
			else if (GetLastError() == ERROR_BROKEN_PIPE) {
				bResult = FALSE;
			}
			else {
				PrintErrorMessage(__FILE__, "ReadFile", __LINE__);
				ExitProcess(1);
			}

		} while (bResult && (nBytesRead > 0));

		CloseHandle(hStdin);
	}

	CloseHandle(hFile);

	ExitProcess(0);
}